from setuptools import setup, find_packages

setup(
    name='paradi',
    version='0.1.1',
    packages=find_packages(),
    install_requires=[
        "requests>=2.31.0"
    ],
    python_requires='>=3.10',
    author='Julien Crambes',
    author_email='julien.crambes@gmail.com',
)



# %% imports
import json
import os
from flask import Flask, abort, request, make_response
from dotenv import load_dotenv

# %% constants
load_dotenv()
with open("http.json", "r") as f:
    http_dict = json.load(f)


def create_app(config: dict = None):
    app_instance = Flask(__name__)
    app_instance.config.from_object(config)
    return app_instance


app = create_app()


def shutdown_server():
    func = request.environ.get('werkzeug.server.shutdown')
    if func is None:
        raise RuntimeError('Not running with the Werkzeug Server')
    func()


# %% Routing


@app.route("/")
def index():
    return make_response(http_dict["200"], 200)


@app.route("/http/<int:status_code>", methods=["GET", "POST", "PUT", "PATCH", "DELETE"])
def http_code_check(status_code: int):
    if status_code >= 400:
        try:
            abort(status_code)
        except LookupError:
            abort(500)

    return make_response(http_dict[str(status_code)], status_code)


@app.route("/post/<string:input_type>", methods=["GET", "POST"])
def post(input_type: str):
    response = make_response(f"{input_type} post successfully", 200)
    response.content_type = "application/json"

    match input_type:
        case "data":
            response.data = request.data
        case "args":
            response.data = json.dumps(request.args)
        case "form":
            response.data = json.dumps(request.form)
        case "files":
            files_submitted = {filename: file.content_length for filename, file in request.files.items()}
            response.data = json.dumps(files_submitted)
        case "values":
            data_submitted = {"args": request.args,
                              "form": request.form}
            response.data = json.dumps(data_submitted)
        case "json":
            response.data = request.json
        case _:
            abort(400)

    return response


@app.route('/shutdown')
def shutdown():
    shutdown_server()
    return 'Server shutting down...'


if __name__ == "__main__":
    app.run(host=os.getenv("TEST_SERVER_HOSTNAME"),
            port=os.getenv("TEST_SERVER_PORT"),
            debug=True,
            use_reloader=False)



# %% imports
import pytest
import requests
import json
import os
from paradi import Paradi
from dotenv import load_dotenv


# %% constants
load_dotenv()
with open("http.json", "r") as f:
    http_dict = json.load(f)


# %% fixtures
@pytest.fixture
def test_paradi_instance():
    class TestParadiClass(Paradi):
        def __init__(self, **kwargs):
            super().__init__(**kwargs)

        def _save_auth(self,
                       response: requests.Response
                       ):
            if response.status_code == 200:
                self.auth = {"auth_saved": True}

    return TestParadiClass(entry=f"http://{os.getenv('TEST_SERVER_HOSTNAME')}:{os.getenv('TEST_SERVER_PORT')}",
                           login_uri="",
                           logout_uri="",
                           login_kwargs={})


# %% Tests
class TestParadi:
    def test_instantiate(self):
        with pytest.raises(TypeError):
            Paradi.__new__(Paradi)

    def test__request(self,
                      test_paradi_instance):
        with pytest.raises(AttributeError):
            test_paradi_instance.__request("GET", "200")

    def test_get(self,
                 test_paradi_instance):
        assert isinstance(test_paradi_instance.get("http/200"), requests.Response)
        for status_code, message in http_dict.items():
            if 200 <= int(status_code) < 300:
                assert test_paradi_instance.get(f"http/{status_code}").status_code == int(status_code)
            else:
                with pytest.raises(Exception):
                    test_paradi_instance.get(f"http/{status_code}")

    def test_post(self,
                  test_paradi_instance):
        test_request = test_paradi_instance.post("post/data")
        assert isinstance(test_request, requests.Response)
        assert test_request.status_code == 200

    def test__save_auth(self,
                        test_paradi_instance):
        response = test_paradi_instance.post("post/data")
        test_paradi_instance._save_auth(response=response)
        assert test_paradi_instance.auth is not None
